@@ -1,3 +1,4 @@
+import threading
 from .wordtile import WordTile
 import pygame
 from typing import Tuple, List
@@ -12,8 +13,8 @@ class Console:
     width: int
     height: int
     font: pygame.font.Font
-    font_width: int
-    font_height: int
+    text_width: int
+    text_height: int
     window_width: int
     window_height: int
     data: List[List[WordTile]]
@@ -23,6 +24,8 @@ class Console:
     is_open: bool
     cursor_x: int
     cursor_y: int
+    cursor_bkcolor: pygame.Color
+    enabled_cursor_twinkling: bool
     cursor_visible: bool
     cursor_tick: int
     cursor_last_change_time: int
@@ -32,6 +35,7 @@ class Console:
     height_offset: int
     fps_last_update_time: int
     fps_cnt: int
+    buffer_surface: pygame.Surface
 
     def __init__(self):
         """
@@ -53,10 +57,10 @@ class Console:
         self.height_offset = 0
         pygame.init()
         self.font = pygame.font.Font(os.path.dirname(os.path.abspath(__file__)) + r'\Apple Hei.ttf', font_height)
-        [self.font_width, self.font_height] = self.font.size('a')
-        self.font_height -= self.height_offset * 2
-        [self.window_width, self.window_height] = [self.width * self.font_width,
-                                                   self.height * self.font_height]
+        [self.text_width, self.text_height] = self.font.size('a')
+        self.text_height -= self.height_offset * 2
+        [self.window_width, self.window_height] = [self.width * self.text_width,
+                                                   self.height * self.text_height]
         self.data = [[WordTile(' ', pygame.Color('#39C5BB'), pygame.Color('white')) for i in range(width)] for j in
                      range(height)]
         self.data_updated = True
@@ -66,15 +70,64 @@ class Console:
         self.is_open = True
         self.cursor_x = 0
         self.cursor_y = 0
+        self.cursor_bkcolor = self.data[self.cursor_x][self.cursor_y].back_color
+        self.enabled_cursor_twinkling = True
         self.cursor_visible = True
         self.cursor_tick = 500
         self.screen_last_update_time = pygame.time.get_ticks()
         self.cursor_last_change_time = pygame.time.get_ticks()
         self.cursor_width = 2
-        self.fps = 120
+        self.fps = 60
         self.fps_last_update_time = pygame.time.get_ticks()
         self.fps_cnt = 0
 
+    def __render_single_char(self, i, j) -> None:
+        """
+        渲染单个字符
+        :param i: 字符的第一坐标
+        :param j: 字符的第二坐标
+        """
+        obj = self.data[i][j]
+        [x, y] = [j * self.text_width, i * self.text_height]
+        if obj.wide:
+            pygame.draw.rect(self.window, obj.back_color, (x, y, self.text_width * 2, self.text_height))
+        else:
+            pygame.draw.rect(self.window, obj.back_color, (x, y, self.text_width, self.text_height))
+        text = self.font.render(obj.char, True, obj.fore_color, obj.back_color)
+        self.window.blit(text.subsurface(pygame.Rect(0, self.height_offset, self.text_width,
+                                                     self.text_height - self.height_offset * 2)), (x, y))
+
+    def __render_cursor(self) -> None:
+        pygame.draw.line(self.window, pygame.color.Color('black') if self.cursor_visible else self.cursor_bkcolor,
+                         (self.cursor_y * self.text_width, self.cursor_x * self.text_height),
+                         (self.cursor_y * self.text_width,
+                          self.cursor_x * self.text_height + self.text_height - 1), self.cursor_width)
+
+    def __render(self) -> None:
+        """
+        渲染
+        """
+        if self.data_updated:
+            self.data_updated = False
+            for i, row in enumerate(self.data):
+                """render"""
+                wide = False
+                for j, obj in enumerate(row):
+                    skip = False
+                    if not obj.is_new:
+                        skip = True
+                    obj.is_new = False
+                    if wide:
+                        wide = False
+                        skip = True
+                    if obj.wide:
+                        wide = True
+                    if not skip:
+                        self.__render_single_char(i, j)
+        if self.enabled_cursor_twinkling:
+            self.__render_cursor()
+        pygame.display.flip()
+
     def start(self) -> None:
         """
         开启控制台的事件监听和更新操作，请在一个独立的线程执行 init 和此方法。
@@ -84,47 +137,16 @@ class Console:
             if (now - self.screen_last_update_time) * self.fps >= 1000:
                 self.fps_cnt += 1
                 self.screen_last_update_time = now
-                if now - self.cursor_last_change_time >= self.cursor_tick:
-                    self.cursor_visible = not self.cursor_visible
-                    self.cursor_last_change_time = now
-                if self.data_updated:
-                    self.data_updated = False
-                    for i, row in enumerate(self.data):
-                        """render"""
-                        wide = False
-                        for j, obj in enumerate(row):
-                            skip = False
-                            if not obj.is_new:
-                                skip = True
-                            if wide:
-                                wide = False
-                                skip = True
-                            if skip:
-                                continue
-                            obj.is_new = False
-                            [x, y] = [j * self.font_width, i * self.font_height]
-                            if obj.wide:
-                                wide = True
-                                pygame.draw.rect(self.window, obj.back_color,
-                                                 (x, y, x + self.font_width * 2, y + self.font_height))
-                            else:
-                                pygame.draw.rect(self.window, obj.back_color,
-                                                 (x, y, x + self.font_width, y + self.font_height))
-                            text = self.font.render(obj.char, True, obj.fore_color, obj.back_color)
-                            self.window.blit(text.subsurface(pygame.Rect(0, self.height_offset, text.get_width(),
-                                                                         text.get_height() - self.height_offset * 2)),
-                                             (x, y))
-                cursor_color = pygame.color.Color('black') if self.cursor_visible else self.data[self.cursor_x][self.cursor_y].back_color
-                pygame.draw.line(self.window, cursor_color,
-                                 (self.cursor_y * self.font_width, self.cursor_x * self.font_height),
-                                 (self.cursor_y * self.font_width,
-                                  self.cursor_x * self.font_height + self.font_height - 1), self.cursor_width)
-                pygame.display.flip()
+                self.__render()
+
+            if now - self.cursor_last_change_time >= self.cursor_tick:
+                self.cursor_visible = not self.cursor_visible
+                self.cursor_last_change_time = now
 
             if now - self.fps_last_update_time >= 1000:
                 print('fps:', self.fps_cnt * 1000 / (now - self.fps_last_update_time))
                 self.fps_cnt = 0
-                self.fps_last_update_time = pygame.time.get_ticks()
+                self.fps_last_update_time = now
 
             for event in pygame.event.get():
                 if event.type == pygame.QUIT:
@@ -143,9 +165,14 @@ class Console:
         :param y: y 坐标
         :param tile: 一个 WordTile 对象，表示一个文字
         """
-        self.data_updated = True
         self.data[x][y] = tile
 
+    def flush(self) -> None:
+        """
+        刷新控制台界面
+        """
+        self.data_updated = True
+
     def update_cursor_pos(self, x, y) -> None:
         """
         修改控制台光标的位置，一般由 mocio.Printer 对象代理，不需要手动操作。
@@ -153,7 +180,24 @@ class Console:
         :param x: x 坐标
         :param y: y 坐标
         """
+        self.cursor_visible = False
+        self.__render_cursor()
         self.cursor_x = x
         self.cursor_y = y
+        self.cursor_bkcolor = self.data[self.cursor_x][self.cursor_y].back_color
+        # 理论需要加锁同步，但是光标 bug 影响不大
         self.cursor_visible = True
         self.cursor_last_change_time = pygame.time.get_ticks()
+
+    def enable_cursor_twinkling(self) -> None:
+        """
+        启用光标闪烁
+        """
+        self.enabled_cursor_twinkling = True
+
+    def disable_cursor_twinkling(self) -> None:
+        """
+        关闭光标闪烁，同样没有同步，摆烂了（
+        """
+        self.enabled_cursor_twinkling = False
+        self.__render_single_char(self.cursor_x, self.cursor_y)
